<table>
  <tr>
    <th>
      Bundle streams
    </th>
    <td>
    {% for stream in summary_data.bundle_streams.all %}
        {{stream.pathname}}{% if not forloop.last %}, {% endif %}
    {% endfor %}
    </td>
  </tr>
{% if summary_data.attributes %}
  <tr>
    <th>
      Attributes
    </th>
    <td>
    {% for a in summary_data.attributes %}
    {{ a.0 }} == {{ a.1 }} <br />
    {% endfor %}
    </td>
  </tr>
{% endif %}
{% if summary_data.build_number_attribute %}
  <tr>
    <th>
      Build Number Attribute
    </th>
    <td>
      {{ summary_data.build_number_attribute }}
    </td>
  </tr>
{% endif %}
  <tr>
    <th>
      Test cases
    </th>
    <td>
      <table>
        <tbody>
          {% for test in summary_data.tests %}
          <tr>
            <td>
              {{ test.test }}
            </td>
            <td>
              {% for test_case in test.all_case_names %}
              {{ test_case }}
              {% empty %}
              <i>any</i>
              {% endfor %}
            </td>
          </tr>
          {% endfor %}
        </tbody>
      </table>
    </td>
  </tr>
</table>
